<?php

/**
 * Provides a plugin for connecting to a SearchStax Solr server.
 */
class SearchStaxSolrConnection extends SearchApiSolrConnection {

  /**
   * The "magic" parameter used to switch Solr requests to a different handler.
   */
  public const OVERRIDE_HANDLER_PARAM = '_searchstax_override_handler';

  /**
   * Cached Solr version.
   *
   * @see getSolrVersion()
   */
  protected string $solrVersion;

  /**
   * {@inheritdoc}
   */
  protected function constructUrl($servlet, array $params = [], $added_query_string = NULL): string {
    if ($servlet === self::SEARCH_SERVLET) {
      // Add a match-all "q" parameter in case none is passed.
      if (empty($params['q']) && strpos("&$added_query_string", '&q=') === FALSE) {
        $params['q'] = '*:*';
      }
      $handler_param = self::OVERRIDE_HANDLER_PARAM;
      if (!empty($params[$handler_param])) {
        $servlet = $params[$handler_param];
        unset($params[$handler_param]);
      }
      elseif (preg_match("/(^|&)$handler_param=([^&]+)/", $added_query_string, $match)) {
        $servlet = rawurldecode($match[2]);
        $added_query_string = str_replace($match[0], '', $added_query_string);
        if ($match[1] === '') {
          $added_query_string = ltrim($added_query_string, '&');
        }
      }
    }
    return parent::constructUrl($servlet, $params, $added_query_string);
  }

  /**
   * {@inheritdoc}
   */
  protected function makeHttpRequest($url, array $options = []): object {
    // Add "Authorization" header in case token authentication is enabled.
    if (!empty($this->options['update_token'])) {
      $options['headers']['Authorization'] = "Token {$this->options['update_token']}";
    }

    return parent::makeHttpRequest($url, $options);
  }

  /**
   * {@inheritdoc}
   */
  public function getSolrVersion(): string {
    if (!isset($this->solrVersion)) {
      try {
        $this->solrVersion = parent::getSolrVersion();
      }
      catch (Exception $e) {
        // Some SearchStax apps (specifically those that use token authentication,
        // but probably safer not to hard-code that) block access to the admin
        // endpoints. Fall back to a safe Solr version for those.
        $this->solrVersion = '8.11.1';
      }
    }
    return $this->solrVersion;
  }

}
