<?php

/**
 * Provides a Solr service class specifically for SearchStax servers.
 */
class SearchStaxSolrService extends SearchApiSolrService {

  /**
   * The connection class used by this service.
   *
   * Must implement SearchApiSolrConnectionInterface.
   *
   * @var string
   */
  protected $connection_class = SearchStaxSolrConnection::class;

  /**
   * {@inheritdoc}
   */
  public function configurationForm(array $form, array &$form_state): array {
    $form['update_endpoint'] = [
      '#type' => 'textfield',
      '#title' => t('Endpoint'),
      '#description' => t('Just copy &amp; paste the “Update endpoint” value of the SearchStax app / index as shown in your SearchStax account.'),
      '#default_value' => $this->options['update_endpoint'] ?? '',
      '#required' => TRUE,
    ];

    $form['update_token'] = [
      '#type' => 'textfield',
      '#title' => t('Read &amp; write token key'),
      '#description' => t('In case the search app uses token authentication, copy &amp; paste the “Read & Write token key” value of the SearchStax app / index as shown in your SearchStax account.'),
      '#default_value' => $this->options['update_token'] ?? '',
    ];

    $form = parent::configurationForm($form, $form_state);

    $form['scheme'] = [
      '#type' => 'value',
      '#value' => 'https',
    ];
    $form['host'] = [
      '#type' => 'value',
      '#value' => '',
    ];
    $form['port'] = [
      '#type' => 'value',
      '#value' => '443',
    ];
    $form['path'] = [
      '#type' => 'value',
      '#value' => '/',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function configurationFormValidate(array $form, array &$values, array &$form_state): void {
    parent::configurationFormValidate($form, $values, $form_state);

    $values['update_endpoint'] = trim($values['update_endpoint']);
    $values['update_token'] = trim($values['update_token']);
    if (preg_match('#https://([^/:]+)(/[^/]+/[^/]+)/(update|select)$#', $values['update_endpoint'], $matches)) {
      $values['host'] = $matches[1];
      $values['path'] = $matches[2];
    }
    else {
      form_error($form['update_endpoint'], t('Invalid endpoint format'));
    }
  }

}
